/*
 * Turbo Cable Utilities
 *
 * Copyright (C) 2004, 2005 BLADOX, s.r.o.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 */


#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <time.h>
#include <stdarg.h>
#include "modeminit.h"
#include "logging.h"
#include "alarm.h"

#include <cable.h>
#include <err.h>

#include "cablelib.h"
#include "extras.h"

u8 cable_sms_head[] = "029100110000810004FF";
u8 cable_sms_head_pdu[] = "110000810004FF";

u8 verbose;
u8 overwrite;
u8 sort;

u16 fs_ino_block (u32 b)
{
  u16 x = (b & 0x0000FFFF);

  return x;
}

u8 fs_ino_index (u32 b)
{
  u8 x = (b >> 16);

  return x;
}

u32 fs_ino_p (u16 p, u8 i)
{
  u32 x;

  x = ((u32) i << 16) | p;
  return x;
}

b8 ch_write (u8 * s, u32 l)
{
// RETS BUF
  u8 buf[1000];
  u8 cmd[1000];
  u8 answer[1000];
  u8 *res;
  u32 i, j;

  i = sprintf (buf, "%s%02X", cable_sms_head, l);
  for (j = 0; j < l; j++)
    i += sprintf (buf + i, "%02X", s[j]);
  i += sprintf (buf + i, "\x1A");
  printf ("SEND: %02X %s\n", l, buf); fflush(stdout);
  sprintf (cmd, "AT+CMGW=%d\r", l + strlen (cable_sms_head_pdu) / 2 + 1);
  put_command (cmd, answer, sizeof (answer), 50, ">");
  printf ("ANS %s\n", answer); fflush(stdout);
  put_command (buf, answer, sizeof (answer), 200, 0);
  printf ("ANS %s\n", answer); fflush(stdout);
  if (strstr (answer, "ERROR"))
    return -1;
  for (i = 1; i < 11; i++)
  {
    sprintf (buf, "+CMGW: %i", i);
    res = strstr (answer, buf);
    if (res)
    {
      printf ("AT POS %d\n", i); fflush(stdout);
      return i;
    }
  }
  return 0;
}

u8 hex_2_u4 (u8 * s)
{
  u8 v = *s;

  if ('0' <= v && v <= '9')
  {
    v = v - '0';
    return v;
  }
  if ('A' <= v && v <= 'F')
  {
    v = v - 'A' + 10;
    return v;
  }
  return 0;
}

u8 hex_2_u8 (u8 * s)
{
  u8 a;
  u8 b;

  a = hex_2_u4 (s);
  b = hex_2_u4 (s + 1);
//printf("A %d B %d\n",a,b);
  return (a << 4) | b;
}

u8 ch_read (u8 i, u8 * s)
{
  u8 j;
  u8 l = strlen (cable_sms_head);
  u8 *res;
  u8 cmd[500];
  u8 answer[500];

// RETS LEN
  sprintf (cmd, "AT+CMGR=%d\r", i);
  put_command (cmd, answer, sizeof (answer), 200, 0);
  printf ("ANS %s\n", answer);  fflush(stdout);
  res = strstr (answer, "+CMGR:");
  res = strstr (res, "\n");
/*
for(j=0;j<10;j++)printf("X %02X\n",res[j]);
printf("R%s\n",res);
*/
  res++;
  res += l;

//printf("R1 %d %s\n",res, res);
  printf ("READ: %s\n", res); fflush(stdout);
  l = hex_2_u8 (res);
  res += 2;
  for (j = 0; j < l; j++)
  {
    s[j] = hex_2_u8 (res);
//      printf("X %02X\n",s[j]);
    res += 2;
  }

  sprintf (cmd, "AT+CMGD=%d\r", i);
  put_command (cmd, answer, sizeof (answer), 200, 0);
  printf ("ANS %s\n", answer);  fflush(stdout);
  return l;
}

b8 app_open ()
{
  u8 i, j;
  u8 *t;
  u8 res[1000];
  b32 f;
b8 err;

  i = NO_ERROR;

  i = 0;
  t = res;
  *t = CABLE_UPLOAD_APP;
  t++;
  *t = CABLE_UPLOAD_APP_START;
  t++;

  err = ch_write (res, t - res);
  if (err>0)
  {
    err = ch_read (err, res);
    err = res[0];
  }
  return err;
}

b8 app_append (u8 * data, u16 pos, u8 l)
{
  u8 i = NO_ERROR;
  u8 res[1000];
  u8 *t = res;
b8 err;

  *t = CABLE_UPLOAD_APP;
  t++;
  *t = CABLE_UPLOAD_APP_CONT;
  t++;
  *t = l;
  t++;
  for (i = 0; i < l; i++)
  {
    *t = data[pos + i];
    t++;
  }
  err = ch_write (res, t - res);
  if (err>0)
  {
    err = ch_read (err, res);
    err= res[0];
  }
  return err;
}

b8 app_close (u8 * name)
{
  u8 i = NO_ERROR;
  u8 res[1000];
  u8 *t = res;
  u8 l = strlen (name);
b8 err;

  *t = CABLE_UPLOAD_APP;
  t++;
  *t = CABLE_UPLOAD_APP_END;
  t++;

  for (i = 0; i < l; i++)
  {
    *t = name[i];
    t++;
  }
  *t = '\0';
  t++;
  err = ch_write (res, t - res);
  if (err>0)
  {
    err = ch_read (err, res);
    err = res[0];
  }
  return err;
}

b32 create_file (u8 * path, u8 * name, u8 * ext, u8 srt)
{
  u8 i, j;
  u8 *t;
  u8 res[1000];
  b32 f;

  i = 0;
  t = res;
  *t = CABLE_CREATE_FILE;
  t++;
  *t = srt;
  t++;
  for (i = 0; i < strlen (path); i++)
  {
    *t = path[i];
    t++;
  }
  *t = '\0';
  t++;
  for (i = 0; i < strlen (name); i++)
  {
    *t = name[i];
    t++;
  }
  *t = '\0';
  t++;

  for (i = 0; i < 3; i++)
  {
    if (ext)
      *t = ext[i];
    else
      *t = 0;
    t++;
  }

  i = ch_write (res, t - res);
  if (i)
  {
    i = ch_read (i, res);
//      for(j=0;j<i;j++)printf("Y %02X\n",res[j]);
    LOAD32L (f, res);
    printf ("FILE: %08X\n", f); fflush(stdout);
    return f;
  }
  return 0;
}

u8 file_append (u32 f, u8 * data, u16 pos, u8 l)
{
  u8 i;
  u8 res[1000];
  u8 *t = res;

  printf ("file_append f=%d, pos=%d, len=%d\n", f, pos, l); fflush(stdout);
  *t = CABLE_FILE_APPEND;
  t++;
  STORE32L (f, t);
  t += 4;
  *t = l;
  t++;
  for (i = 0; i < l; i++)
  {
    *t = data[pos + i];
    t++;
  }
  i = ch_write (res, t - res);
  if (i)
  {
    i = ch_read (i, res);
  }
  return 0;
}

b32 create_dir (u8 * path, u8 * name, u8 srt)
{
  u8 i, j;
  u8 *t;
  u8 res[1000];
  b32 f;

  i = 0;
  t = res;
  *t = CABLE_CREATE_DIR;
  t++;
  *t = srt;
  t++;
  for (i = 0; i < strlen (path); i++)
  {
    *t = path[i];
    t++;
  }
  *t = '\0';
  t++;
  for (i = 0; i < strlen (name); i++)
  {
    *t = name[i];
    t++;
  }
  *t = '\0';
  t++;

  i = ch_write (res, t - res);
  if (i)
  {
    i = ch_read (i, res);
//      for(j=0;j<i;j++)printf("Y %02X\n",res[j]);
    LOAD32L (f, res);
    printf ("DIR: %08X\n", f); fflush(stdout);
    return f;
  }
  return 0;
}

/*
void format_df()
{
u8 i,j;
u8 *t;
u8 res[1000];
b32 f;

i=0;
t=res;
*t=CABLE_FORMAT; t++;

i=ch_write(res,t-res);
if(i)
{
	i=ch_read(i,res);
}
}
*/

b32 find_file (u8 * path, u8 * name, u32 * len, u32 * len_raw)
{
  u8 i, j;
  u8 *t;
  u8 res[1000];
  b32 f;

  i = 0;
  t = res;
  *t = CABLE_FILE_FIND;
  t++;
  for (i = 0; i < strlen (path); i++)
  {
    *t = path[i];
    t++;
  }
  *t = '\0';
  t++;
  for (i = 0; i < strlen (name); i++)
  {
    *t = name[i];
    t++;
  }
  *t = '\0';
  t++;

  i = ch_write (res, t - res);
  if (i)
  {
    i = ch_read (i, res);
//      for(j=0;j<i;j++)printf("Y %02X\n",res[j]);
    LOAD32L (f, res);
    printf ("FILE: %08X %d\n", f, f); fflush(stdout);
    LOAD32L (*len, res + 4);
    printf ("LEN: %d\n", *len); fflush(stdout);
    LOAD32L (*len_raw, res + 8);
    printf ("LEN_RAW: %d\n", *len); fflush(stdout);
    return f;
  }
  return 0;
}

u8 file_read (u32 f, u8 * data, u32 pos, u8 l)
{
  u8 i;
  u8 res[1000];
  u8 *t = res;

  printf ("file_read f=%d, pos=%d, len=%d\n", f, pos, l); fflush(stdout);
  *t = CABLE_FILE_READ;
  t++;
  STORE32L (f, t);
  t += 4;
  *t = l;
  t++;
  STORE32L (pos, t);
  t += 4;
  i = ch_write (res, t - res);
  if (i)
  {
    i = ch_read (i, res);
    for (i = 0; i < l; i++)
      data[i] = res[i];
  }
  return 0;
}

u8 detect_mc (u32 * root, u8 * df_id, u8 * df_blocks, u16 * df_pages)
{
  u8 i;
  u8 res[1000];
  u8 *t = res;
  u8 ret = 0xFF;

  *t = CABLE_MC_INFO;
  t++;
  i = ch_write (res, t - res);
  if (i)
  {
    i = ch_read (i, res);
    ret = res[0];		//status
    *df_id = res[1];
    *df_blocks = res[2];
    *df_pages = res[3] | (res[4] << 8);
    LOAD32L (*root, &res[5]);
  }
  return ret;
}

u8 block_read (u8 * data, u16 block, u8 pos, u8 l)
{
  u8 i;
  u8 res[1000];
  u8 *t = res;

  *t = CABLE_READ_BLOCK;
  t++;
  *t = block;
  t++;
  *t = block >> 8;
  t++;
  *t = pos;
  t++;
  *t = l;
  t++;
  i = ch_write (res, t - res);
  if (i)
  {
    i = ch_read (i, res);
    for (i = 0; i < l; i++)
      data[i] = res[i];
  }
  return 0;
}

b8 cable_delete_inode (u32 f)
{
  u8 i, j;
  u8 *t;
  u8 res[1000];
b8 err;

  i = 0;
  t = res;
  *t = CABLE_DELETE_INODE;
  t++;
  STORE32L (f, t);
  t += 4;

  err = ch_write (res, t - res);
  if (err>0)
  {
    err = ch_read (err, res);
    err = res[0];
  }
return err;
}

b8 cable_clear_apps ()
{
  u8 i, j;
  u8 *t;
  u8 res[1000];
b8 err;

  i = 0;
  t = res;
#ifndef CABLE_CLEAR_APPS
#define CABLE_CLEAR_APPS 0x0d
#endif
  *t = CABLE_CLEAR_APPS;
  t++;

  err = ch_write (res, t - res);
  if (err>0)
  {
    err = ch_read (err, res);
    err = res[0];
  }
return err;
}

b8 cable_reset ()
{
  u8 i, j;
  u8 *t;
  u8 res[1000];
b8 err;

  i = 0;
  t = res;
#ifndef CABLE_RESET_DEFAULT
#define CABLE_RESET_DEFAULT 0x0e
#endif
  *t = CABLE_RESET_DEFAULT;
  t++;

  err = ch_write (res, t - res);
  if (err>0)
  {
    err = ch_read (err, res);
    err = res[0];
  }
return err;
}

b8 cable_info ()
{
  u8 i, j,k,l;
  u8 *t;
  u8 res[1000];
b8 err;

  i = 0;
  t = res;
#ifndef CABLE_INFO
#define CABLE_INFO 0x10
#endif
  *t = CABLE_INFO;
  t++;

  err = ch_write (res, t - res);
  if (err>0)
  {
    err = ch_read (err, res);
    j=err;
    err = res[0];
if(err==NO_ERROR)
{
i=1;
while(i<j)
{
#ifndef CABLE_INFO_VERSION
#define CABLE_INFO_VERSION      0x01
#endif

if(res[i]==CABLE_INFO_VERSION)
{
i++;
printf("Kernel Version %d.%d.%d.%d\n",res[i+1], res[i+2], res[i+3], res[i+4]);
i+=res[i]+1;
}

#ifndef CABLE_INFO_SN
#define CABLE_INFO_SN           0x02
#endif

if(res[i]==CABLE_INFO_SN)
{
i++;
printf("Serial Number ");
l=res[i];
for(k=0;k<l;k++)printf("%c",res[i+1+k]);
printf("\n");
i+=l+1;
}
}
}
  }
return err;
}


b8 cable_lock ()
{
  u8 i, j;
  u8 *t;
  u8 res[1000];
b8 err;

  i = 0;
  t = res;
#ifndef CABLE_LOCK_TURBO
#define CABLE_LOCK_TURBO 0x0f
#endif
  *t = CABLE_LOCK_TURBO;
  t++;

  err = ch_write (res, t - res);
  if (err>0)
  {
    err = ch_read (err, res);
    err = res[0];
  }
return err;
}


u8 verbose = 0;
#ifndef WIN32_MINGW
int baudrate = B19200;
#else
int baudrate = CBR_19200;
#endif

void set_verbose ()
{
  verbose = 1;
}

void set_baudrate (int bd)
{
  switch (bd)
  {
#ifndef WIN32_MINGW
    case 300:
      baudrate = B300;
      break;
    case 1200:
      baudrate = B1200;
      break;
    case 2400:
      baudrate = B2400;
      break;
    case 9600:
      baudrate = B9600;
      break;
    case 19200:
      baudrate = B19200;
      break;
    case 38400:
      baudrate = B38400;
      break;
#ifdef B57600
    case 57600:
      baudrate = B57600;
      break;
#endif
#ifdef B115200
    case 115200:
      baudrate = B115200;
      break;
#endif
#ifdef B230400
    case 230400:
      baudrate = B230400;
      break;
#endif
#else
    case 300:
      baudrate = CBR_300;
      break;
    case 1200:
      baudrate = CBR_1200;
      break;
    case 2400:
      baudrate = CBR_2400;
      break;
    case 9600:
      baudrate = CBR_9600;
      break;
    case 19200:
      baudrate = CBR_19200;
      break;
    case 38400:
      baudrate = CBR_38400;
      break;
#ifdef B57600
    case 57600:
      baudrate = CBR_57600;
      break;
#endif
#ifdef B115200
    case 115200:
      baudrate = CBR_115200;
      break;
#endif
#ifdef B230400
    case 230400:
      baudrate = CBR_230400;
      break;
#endif
#endif
    default:
      writelogfile (_MY_LOG_ERR, "Baudrate not supported");
      exit (1);
  }
}

void print_result(b8 r)
{
//printf("RESULT: ");fflush(stdout);
switch(r)
{
case -1:
	printf("Mobile Phone/Serial Cable Communication Error\n");
	break;
case NO_ERROR:
	printf("OK. No Error\n");
	break;
case ERR_NO_PROGMEM:
	printf("ERROR: Not Enough PROGMEM\n");
	break;
case ERR_NO_RAM:
	printf("ERROR: Not Enough RAM\n");
	break;
case ERR_UNKNOWN:
	printf("ERROR: Unknown Error\n");
	break;
case ERR_NO_APP_SLOT:
	printf("ERROR: No Application Slot\n");
	break;
case ERR_MC_NOT_PRESENT:
	printf("ERROR: Memory Card not present\n");
	break;
case ERR_MC_NOT_READY:
	printf("ERROR: Not Enough EEPROM\n");
	break;
case ERR_BAD_FILE:
	printf("ERROR: Bad File\n");
	break;
#ifndef ERR_TURBO_LOCKED
#define ERR_TURBO_LOCKED                0x48
#endif

case ERR_TURBO_LOCKED:
	printf("ERROR: Turbo Locked\n");
	break;
default:
	printf("Unknown Error\n");
	break;
}
}
