/*
SMS Server Tools
Copyright (C) 2000 Stefan Frings

This program is free software unless you got it under another license directly
from the author. You can redistribute it and/or modify it under the terms of
the GNU General Public License as published by the Free Software Foundation.
Either version 2 of the License, or (at your option) any later version.

http://www.isis.de/members/~s.frings
mailto:s.frings@mail.isis.de
*/

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#ifdef SOLARIS
#include <sys/filio.h>
#endif

#include <stdio.h>
#include <string.h>
#include <unistd.h>

#ifndef WIN32_MINGW
#include <sys/ioctl.h>
#else
#include <windows.h>
#endif

#include "extras.h"
#include "modeminit.h"

int put_command (char *command, char *answer, int max, int timeout,
		 char *expect)
{
  int count = 0;
  int readcount;
  int toread;
  char tmp[100];
  int timeoutcounter = 0;
  int found = 0;
  int available;
  int status;
  char loganswer[2048];

  // send command
  if (command)
    if (command[0])
    {
      // Cycwin does not support TIOC functions and has not workaround for this.
      // So do not check CTS on Cygwin.
#ifndef WINDOWS
      if (rtscts)
      {
	ioctl (modem, TIOCMGET, &status);
	while (!(status & TIOCM_CTS))
	{
	  usleep (100000);
	  timeoutcounter++;
	  ioctl (modem, TIOCMGET, &status);
	  if (timeoutcounter >= timeout)
	  {
	    writelogfile (_MY_LOG_ERR, "Modem is not clear to send");
	    alarm_handler (_MY_LOG_ERR, "", "Modem is not clear to send");
	    return 0;
	  }
	}
      }
#endif
      writelogfile (_MY_LOG_DEBUG, "-> %s", command);
#ifdef WIN32_MINGW
{
DWORD xin;
WriteFile(modem, command, strlen(command), &xin, NULL);
}
#else
      write (modem, command, strlen (command));
      tcdrain (modem);
#endif
    }

  answer[0] = 0;
  do
  {
    // try to read some bytes.

    // Cygwin does not support TIOC functions and has not workaround for this.
    // So do not check number of available bytes
#ifndef WINDOWS
    ioctl (modem, FIONREAD, &available);	// how many bytes are available to read?
    if (available < 1)		// if 0 then wait a little bit and retry
    {
      usleep (100000);
      //write(1,".",1);
      timeoutcounter++;
      ioctl (modem, FIONREAD, &available);
    }
#else
    // Only for Windows. Read as much as possible
    //usleep (100000);
    sleep (1);
    //write(1,".",1);
    timeoutcounter++;
    available = sizeof (tmp) - 1;
#endif
    if (available > 0)
    {
      // How many bytes do I want to read maximum? Not more than tmp buffer size.
      toread = max - count - 1;
      if (toread > sizeof (tmp) - 1)
	toread = sizeof (tmp) - 1;
      // And how many bytes are available?
      if (available < toread)
	toread = available;
      // read data
#ifdef WIN32_MINGW
ReadFile(modem, tmp, toread, (DWORD*)(&readcount), NULL);
#else
      readcount = read (modem, tmp, toread);
#endif
      if (readcount < 0)
	readcount = 0;
      tmp[readcount] = 0;
      // add read bytes to the output buffer
      if (readcount)
      {
	strcat (answer, tmp);
	count += readcount;

	// if we have more time to read, check if we got already the expected string
	if ((timeoutcounter < timeout) && (found == 0))
	{
	  // check if it's the expected answer
	  if ((strstr (answer, "OK\r")) || (strstr (answer, "ERROR")))
	    found = 1;
	  if (expect)
	    if (expect[0])
	      if (strstr (answer, expect))
		found = 1;
	  // if found then set timoutcounter to read only 0.1s after that and not more
	  if (found)
	    timeoutcounter = timeout - 1;
	    //timeoutcounter = timeout;
	}
      }
    }
  }
  // repeat until timout
  while (timeoutcounter < timeout);

  strncpy (loganswer, answer, sizeof (loganswer));
  cutspaces (loganswer);
  cut_emptylines (loganswer);
  writelogfile (_MY_LOG_DEBUG, "<- %s", loganswer);

  return count;
}

void setmodemparams ()		/* setup serial port */
{
#ifndef WIN32_MINGW
  struct termios newtio;

  bzero (&newtio, sizeof (newtio));
//  newtio.c_cflag = baudrate | CRTSCTS | CS8 | CLOCAL | CREAD | O_NDELAY;
  newtio.c_cflag = CRTSCTS | CS8 | CLOCAL | CREAD | O_NDELAY;
  cfsetispeed (&newtio, baudrate);
  cfsetospeed (&newtio, baudrate);
  if (!rtscts)
    newtio.c_cflag &= ~CRTSCTS;
  newtio.c_iflag = IGNPAR;
  newtio.c_oflag = 0;
  newtio.c_lflag = 0;
#ifdef WINDOWS
  newtio.c_cc[VTIME] = 0;
#else
  newtio.c_cc[VTIME] = 1;
#endif
  newtio.c_cc[VMIN] = 0;
  tcflush (modem, TCIOFLUSH);
  tcsetattr (modem, TCSANOW, &newtio);
#else
DCB dcb;

GetCommState(modem, &dcb);
//  dcb.BaudRate = CBR_19200;
  dcb.BaudRate = baudrate;
  dcb.fParity = FALSE;
  dcb.fErrorChar = FALSE; //TR
  dcb.fBinary = TRUE ;
//TR  dcb.fOutxCtsFlow = FALSE;
//  dcb.fOutxCtsFlow = TRUE; //TR
  dcb.fOutxCtsFlow = FALSE; //TR
  dcb.fOutxDsrFlow = FALSE;
//  dcb.fDtrControl = DTR_CONTROL_DISABLE;
  dcb.fDtrControl = DTR_CONTROL_ENABLE;
  dcb.fDsrSensitivity = FALSE;
  dcb.fOutX = FALSE;
  dcb.fInX = FALSE;
//  dcb.fRtsControl = RTS_CONTROL_DISABLE;
  dcb.fRtsControl = RTS_CONTROL_ENABLE;
//  dcb.fRtsControl = RTS_CONTROL_HANDSHAKE; //TR
//  dcb.fRtsControl = RTS_CONTROL_TOGGLE; //TR
//  dcb.XonLim = 1; //TR
//  dcb.XoffLim = 1; //TR

  dcb.fAbortOnError = FALSE;
  dcb.ByteSize = 8;
  dcb.Parity = NOPARITY;
  dcb.StopBits = ONESTOPBIT;
  dcb.fNull=FALSE; // Accept Null Characters 
dcb.fTXContinueOnXoff=TRUE; // XOFf Sent Does Not Stop trans
  SetCommState(modem, &dcb);
#endif
}

void initmodem ()
{
  char command[100];
  char answer[500];
  int retries = 0;
  int success = 0;

printf ("initializing modem\n"); fflush(stdout);

strcpy (command, "ATV1\r");	//Result Code Format
put_command (command, answer, sizeof (answer), 50, 0);
if (strstr (answer, "ERROR"))
{
printf("%s %s",command, answer); fflush(stdout);
}

strcpy (command, "ATE1\r");	//Command Echo
put_command (command, answer, sizeof (answer), 50, 0);
if (strstr (answer, "ERROR"))
{
printf("%s %s",command, answer); fflush(stdout);
}

strcpy (command, "AT+CPMS=\"SM\",\"SM\"\r");	//Preferred Message Store
put_command (command, answer, sizeof (answer), 50, 0);
if (strstr (answer, "ERROR"))
{
printf("%s %s",command, answer); fflush(stdout);
}

strcpy (command, "AT+CMGF=0\r");	//Message Format - 0 PDU format
put_command (command, answer, sizeof (answer), 50, 0);
if (strstr (answer, "ERROR"))
{
printf("%s %s",command, answer); fflush(stdout);
}

strcpy (command, "AT+CPMS?\r");	//Message Memory
put_command (command, answer, sizeof (answer), 50, 0);
// TODO check free
if (strstr (answer, "ERROR"))
{
printf("%s %s",command, answer); fflush(stdout);
}

printf ("modem initiated\n"); fflush(stdout);
}

void openmodem ()
{
#ifdef WIN32_MINGW
COMMTIMEOUTS timeout;
modem=CreateFile(device, GENERIC_READ|GENERIC_WRITE, 0, NULL, OPEN_EXISTING, NULL, NULL);
if(modem==INVALID_HANDLE_VALUE)
{
printf("Cannot open IO port %s (error %d)\n", device, GetLastError()); fflush(stdout);
exit(-1);
}

timeout.ReadIntervalTimeout = 3;
timeout.ReadTotalTimeoutMultiplier = 3;
timeout.ReadTotalTimeoutConstant = 2;
timeout.WriteTotalTimeoutMultiplier = 3;
timeout.WriteTotalTimeoutConstant = 2;
/*
timeout.ReadIntervalTimeout = MAXDWORD;
timeout.ReadTotalTimeoutMultiplier = 0;
timeout.ReadTotalTimeoutConstant = 0;
timeout.WriteTotalTimeoutMultiplier = 10;
timeout.WriteTotalTimeoutConstant = 100;
*/

if(!SetCommTimeouts(modem, &timeout)) {
	printf("Could not set timeout\n");
	exit(-1);
}

#else
  modem = open (device, O_RDWR | O_NOCTTY | O_NONBLOCK);
  if (modem < 0)
  {
    perror (device);
    exit (1);
  }

  tcgetattr (modem, &oldtio);
#endif
}
