/*
 * Turbo Cable Utilities
 *
 * Copyright (C) 2004, 2005 BLADOX, s.r.o.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 */


#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <time.h>
#include <stdarg.h>
#include "modeminit.h"
#include "logging.h"
#include "alarm.h"

#include "cablelib.h"

/*
TODO:
overwrite
if path doesnot exists
error handling
*/

u8 overwrite;
u8 sort;

void help ()
{
  printf ("Usage: [args] src_file dst_file\n");
  printf ("-bx\tbaudrate x (def. 19200)\n");
#ifdef WIN32_MINGW
  printf ("-dx\tdevice x (def. COM1)\n");
#else
  printf ("-dx\tdevice x (def. /dev/ttyS1)\n");
#endif
  printf ("-h\thelp\n");
  printf ("-v\tverbose\n");
  exit (0);
}

int main (int argc, char **argv)
{
  char cmd[500];
  char answer[1000];
  u8 buf[500];
  u8 t[] = { 0x01, 0x23, 0x45, 0x67 };
  u32 i, j;
  b32 f;
  int result;
  struct stat _st;
  FILE *fs;
  u8 *s_name;
  u8 *d_name;
  int s_size;
  u8 *s_buf;
  int ret;
  int pos;
  int len;
  u8 ext[] = { 0, 0, 0, 0 };
  u8 path[1000] = { '/', '\0' };
  u8 name[1000];
  u16 name_end;
  u16 path_end;
  u32 f_len;
  u32 f_len_raw;

#ifdef WIN32_MINGW
  strcpy (device, "COM1");
#else
  strcpy (device, "/dev/ttyS1");
#endif

  rtscts = 1;
  overwrite = 0;
  sort = SORT;

  do
  {
    result = getopt (argc, argv, "vs:od:b:h");
    switch (result)
    {
      case 'H':
	rtscts = 0;
	break;
      case 'd':
	strcpy (device, optarg);
	break;
      case 'b':
	set_baudrate (atoi (optarg));
	break;
      case 'v':
	set_verbose ();
	break;
      case 'h':
	help ();
	break;
    }
  }
  while (result > 0);

  if (argc - optind != 2)
    help ();
  d_name = argv[optind++];
  s_name = argv[optind++];
  printf ("SRC %s\n", d_name);
  printf ("DST %s\n", s_name);

  name_end = strlen (d_name);

  for (i = name_end; i > 0; i--)
    if (d_name[i - 1] == '/')
      break;
  path_end = i;

  len = strlen (d_name);
  for (j = 0; j < name_end - path_end; j++)
    name[j] = d_name[i + j];
  name[j] = '\0';

  for (j = 0; j < path_end; j++)
    path[j] = d_name[j];

  printf ("NAME %s\n", name);
  printf ("PATH %s\n", path);

  openmodem ();
  setmodemparams ();
  initmodem ();

  f = find_file (path, name, &f_len, &f_len_raw);

  pos = 0;
  fs = fopen (s_name, "w+");
  while (pos < f_len)
  {
    len = f_len - pos;
    if (len > 128)
      len = 128;
    file_read (f, buf, pos, len);
    for (i = 0; i < len; i++)
      fputc (buf[i], fs);
    pos += 128;
  }
  fclose (fs);
}
